﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace Věž_z_Lega
{
    class BuildingPlan
    {
        private int x;
        private int y;
        private TowerTypes type;
        private List<Cubes> cubes;
        private List<Layer> layers;
        private string name;
        private string path;


        public BuildingPlan(FileStream fs, string name, string path)
        {
            this.name = name;
            this.path = path;
            using (StreamReader sr = new StreamReader(fs))
            {
                string firstLine = sr.ReadLine();
                string[] firstLineInfos = firstLine.Split(' ');
                this.x = Int32.Parse(firstLineInfos[0]);
                this.y = Int32.Parse(firstLineInfos[1]);
                this.type = (TowerTypes)Enum.Parse(typeof(TowerTypes), firstLineInfos[2]);
                this.cubes = new List<Cubes>();
                this.layers = new List<Layer>();

                string[] lines = sr.ReadToEnd().Split('\n');

                foreach (string line in lines)
                {
                    if (line != String.Empty)
                    {
                        string[] lineInfo = line.Split(' ');
                        int x = Int32.Parse(lineInfo[1].Split('X')[0]);
                        int y = Int32.Parse(lineInfo[1].Split('X')[1]);

                        if (x > y)
                        {
                            int tmp = x;
                            x = y;
                            y = tmp;
                        }

                        this.cubes.Add(new Cubes(x, y, lineInfo[0], Int32.Parse(lineInfo[5])));
                    }
                }

                this.build();
            }
        }

        private void build()
        {
            if (this.type == TowerTypes.D) {

            }
            else if (this.type == TowerTypes.DB) {

            }
            else if (this.type == TowerTypes.P) {
                using (StreamWriter sw = new StreamWriter(new FileStream(this.path + @"\" + this.name + ".45", FileMode.Create)))
                {
                    sw.WriteLine(String.Format("{0} {1} {2}", this.x, this.y, this.type.ToString()));
                bool @continue = true;
                while (@continue) {
                    this.layers.Add(new Layer(this.x, this.y, this.layers.Count + 1));
                    foreach (Cubes cubes in this.cubes)
                    {
                        Tuple<bool, int[]> t = this.layers.Last().TryPlaceP(cubes.ToCube());
                        if (t.Item1)
                        {
                            List<LayerState> states = this.layers.Last().PlaceCube(cubes.ToCube(), t.Item2[1], t.Item2[2]);
                            cubes.Count--;
                            if (states.Contains(LayerState.OK_P)) break;
                            else
                            {
                                int result = this.placeCorrectCube(t.Item2[0], Int32.MaxValue);
                                if (result == -1) @continue = false;
                                if (result == 1) break;
                            }
                        }
                        else
                        {
                            int result = -1;
                            if (t.Item2[1] == 0) {
                                result = this.placeCorrectCube(t.Item2[0], Int32.MaxValue);
                            }
                            else
                            {
                                result = this.placeCorrectCube(Int32.MaxValue, t.Item2[0]);
                            }
                            if (result == -1) @continue = false;
                            if (result == 1) break;
                        }
                    }
                    if (@continue) this.layers.Last().WriteAllCubes(sw);
                }
                
                }
            }
            else {
            }
        }

        private int placeCorrectCube(int requiredX, int requiredY)
        {
            Cubes target = this.cubes.Where(c => c.Count > 0 && ((c.X == requiredX && c.Y == requiredY) || (c.Y == requiredX && c.X == requiredY))).FirstOrDefault();
            if (target == null)
            {
                target = this.cubes.Where(c => c.Count > 0 && ((c.X <= requiredX && c.Y <= requiredY) || (c.Y <= requiredX && c.X <= requiredY))).FirstOrDefault();
                if (target == null) return -1;
            }
                Cube cub;
                if (target.X <= requiredX)
                {
                    cub = target.ToCube();
                }
                else
                {
                    cub = target.ToCube().RotateCube();
                }
                Tuple<bool, int[]> t = this.layers.Last().TryPlaceP(cub);
                if (t.Item1)
                {
                    List<LayerState> states = this.layers.Last().PlaceCube(cub, t.Item2[1], t.Item2[2]);
                    target.Count--;
                    if (states.Contains(LayerState.OK_P)) return 1;
                    else
                    {
                        int result = this.placeCorrectCube(t.Item2[0], Int32.MaxValue);
                        if (result == -1) return -1;
                        if (result == 1) return 1;
                    }
                }
                else
                {
                    int result = -1;
                    if (t.Item2[1] == 0) {
                        result = this.placeCorrectCube(t.Item2[0], Int32.MaxValue);
                    }
                    else
                    {
                        result = this.placeCorrectCube(Int32.MaxValue, t.Item2[0]);
                    }
                    if (result == -1) return -1;
                    if (result == 1) return 1;
                }

                return Int32.MaxValue;
        }

    }
}
