﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;

namespace Věž_z_Lega
{
    class Layer
    {
        public bool[][] Segments { get; set; }
        public int X { get; set; }
        public int Y { get; set; }
        private int filledSegments;
        private int filledLayers;
        public List<CubeInLayer> Cubes { get; set; }
        public int Num { get; set; }

        public Layer(int x, int y, int num)
        {
            this.X = x;
            this.Y = y;
            this.Num = num;
            this.Segments = new bool[x][];
            this.filledSegments = 0;
            this.filledLayers = 0;
            this.Cubes = new List<CubeInLayer>();

            for (int i = 0; i < x; i++)
            {
                this.Segments[i] = new bool[y];
            }
        }

        public Tuple<bool, int[]> TryPlaceP(Cube cube)
        {
            int width = 0;
            bool first = false;
            bool second = false;
            int a = -1, b = this.filledLayers;
            for (int i = 0; i < this.X; i++)
            {
                if (this.Segments[this.filledLayers][i] && !first) first = true;
                else if (this.Segments[this.filledLayers][i] && first && second) break;
                else if (!this.Segments[this.filledLayers][i])
                {
                    if (first) second = true;
                    if (width == 0) a = i;
                    width++;
                }
            }

            if (width >= cube.X && b + cube.Y < this.Y + 1)
            {
                return new Tuple<bool, int[]>(true, new int[3]{width - cube.X, b, a});
            }
            else
            {
                int tmp = (width < cube.X ? 0 : 1); // horizont., jinak vert.
                int requiredLength = (width < cube.X ? width : this.Y - this.filledLayers);
                return new Tuple<bool, int[]>(false, new int[2]{requiredLength, tmp});
            }
        }

        public List<LayerState> PlaceCube(Cube cube, int x, int y)
        {
            int tmpX = (cube.X > cube.Y ? cube.Y : cube.X);
            int tmpY = (cube.X > cube.Y ? cube.X : cube.Y);

            this.Cubes.Add(cube.PlaceCube(x, y));

            if (x + cube.X == this.X)
            {
                this.filledLayers++;
            }

            for (int i = 0; i < cube.Y; i++)
            {
                for (int j = 0; j < cube.X; j++)
                {
                    this.Segments[x + i][y + j] = true;
                    this.filledSegments++;
                }
            }

            List<LayerState> states = new List<LayerState>();

            if (this.filledSegments >= (2 * this.X) + (2 * this.Y) - 4)
            {
                bool ok = true;

                foreach (bool s in this.Segments[0])
                {
                    if (!s)
                    {
                        ok = false;
                        break;
                    }
                }

                for (int i = 1; i < this.Y - 1; i++)
                {
                    if (!Segments[i][0] || !Segments[i][this.X - 1])
                    {
                        ok = false;
                        break;
                    }
                }

                foreach (bool s in this.Segments[this.Y - 1])
                {
                    if (!s)
                    {
                        ok = false;
                        break;
                    }
                }

                if (ok) states.Add(LayerState.OK_D);
            }

            if (this.filledSegments == this.X * this.Y)
            {
                states.Add(LayerState.OK_P);
            }

            return states;
        }

        public void WriteAllCubes(StreamWriter sw)
        {
            foreach (CubeInLayer c in this.Cubes)
            {
                sw.WriteLine(String.Format("{0} {1} {2} {3} {4}x{5}", this.Num, c.XPos + 1, c.YPos + 1, c.GetColor(), c.X, c.Y));
            }
        }
    }
}
